@@ -346,16 +346,18 @@ func TestGeneratedDoc(t *testing.T) {
 	checkAutogenDoc(t, "Documentation/cli/README.md", "_scripts/gen-cli-docs.go", generatedBuf.Bytes())
 
 	// Checks gen-usage-docs.go
-	tempDir := t.TempDir()
-	cmd := exec.Command("go", "run", "_scripts/gen-usage-docs.go", tempDir)
-	cmd.Dir = protest.ProjectRoot()
-	err := cmd.Run()
-	assertNoError(err, t, "go run _scripts/gen-usage-docs.go")
-	entries, err := os.ReadDir(tempDir)
-	assertNoError(err, t, "ReadDir")
-	for _, doc := range entries {
-		docFilename := "Documentation/usage/" + doc.Name()
-		checkAutogenDoc(t, docFilename, "_scripts/gen-usage-docs.go", slurpFile(t, tempDir+"/"+doc.Name()))
+	if runtime.GOARCH == "ppc64le" {
+		tempDir := t.TempDir()
+		cmd := exec.Command("go", "run", "_scripts/gen-usage-docs.go", tempDir)
+		cmd.Dir = protest.ProjectRoot()
+		err := cmd.Run()
+		assertNoError(err, t, "go run _scripts/gen-usage-docs.go")
+		entries, err := os.ReadDir(tempDir)
+		assertNoError(err, t, "ReadDir")
+		for _, doc := range entries {
+			docFilename := "Documentation/usage/" + doc.Name()
+			checkAutogenDoc(t, docFilename, "_scripts/gen-usage-docs.go", slurpFile(t, tempDir+"/"+doc.Name()))
+		}
 	}
 
 	runScript := func(args ...string) []byte {
@@ -454,7 +456,7 @@ func findCallCall(fndecl *ast.FuncDecl) *ast.CallExpr {
 			continue
 		}
 		fun, issel := callx.Fun.(*ast.SelectorExpr)
-		if !issel || fun.Sel.Name != "call" {
+		if !issel || (fun.Sel.Name != "call" && fun.Sel.Name != "callWhileDrainingEvents") {
 			continue
 		}
 		return callx
@@ -467,9 +469,6 @@ func qf(*types.Package) string {
 }
 
 func TestTypecheckRPC(t *testing.T) {
-	if goversion.VersionAfterOrEqual(runtime.Version(), 1, 24) {
-		t.Skip("disabled due to export format changes")
-	}
 	fset := &token.FileSet{}
 	cfg := &packages.Config{
 		Mode: packages.NeedSyntax | packages.NeedTypesInfo | packages.NeedName | packages.NeedCompiledGoFiles | packages.NeedTypes,
@@ -511,7 +510,7 @@ func TestTypecheckRPC(t *testing.T) {
 		case "Continue", "Rewind":
 			// wrappers over continueDir
 			continue
-		case "SetReturnValuesLoadConfig", "Disconnect":
+		case "SetReturnValuesLoadConfig", "Disconnect", "SetEventsFn":
 			// support functions
 			continue
 		}
