@@ -231,7 +231,7 @@ func initialize(dbp *nativeProcess) (string, error) {
 		comm = bytes.TrimSuffix(comm, []byte("\n"))
 	}
 
-	if comm == nil || len(comm) <= 0 {
+	if len(comm) <= 0 {
 		stat, err := os.ReadFile(fmt.Sprintf("/proc/%d/stat", dbp.pid))
 		if err != nil {
 			return "", fmt.Errorf("could not read proc stat: %v", err)
@@ -261,7 +261,7 @@ func (dbp *nativeProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
 
 // kill kills the target process.
 func (procgrp *processGroup) kill(dbp *nativeProcess) error {
-	if dbp.exited {
+	if ok, _ := dbp.Valid(); !ok {
 		return nil
 	}
 	if !dbp.threads[dbp.pid].Stopped() {
@@ -563,9 +563,23 @@ func trapWaitInternal(procgrp *processGroup, pid int, options trapWaitOptions) (
 			}
 			// do the same thing we do if a thread quit
 			if wpid == dbp.pid {
+				exitStatus := 0
+				if procgrp.numValid() == 1 {
+					// try to recover the real exit status using waitpid
+					for {
+						wpid2, status2, err := dbp.wait(-1, sys.WNOHANG)
+						if wpid2 <= 0 || err != nil {
+							break
+						}
+						if status2.Exited() {
+							exitStatus = status2.ExitStatus()
+						}
+					}
+
+				}
 				dbp.postExit()
 				if procgrp.numValid() == 0 {
-					return nil, proc.ErrProcessExited{Pid: wpid, Status: status.ExitStatus()}
+					return nil, proc.ErrProcessExited{Pid: wpid, Status: exitStatus}
 				}
 				continue
 			}
@@ -680,7 +694,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	}
 
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		for _, th := range dbp.threads {
@@ -693,8 +707,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	for {
 		th, err := trapWaitInternal(procgrp, -1, trapWaitNohang)
 		if err != nil {
-			p := procgrp.procForThread(th.ID)
-			return nil, exitGuard(p, procgrp, err)
+			return nil, exitGuard(procgrp.procs[0], procgrp, err)
 		}
 		if th == nil {
 			break
@@ -703,7 +716,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 
 	// stop all threads that are still running
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		for _, th := range dbp.threads {
@@ -724,7 +737,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	for {
 		allstopped := true
 		for _, dbp := range procgrp.procs {
-			if dbp.exited {
+			if ok, _ := dbp.Valid(); !ok {
 				continue
 			}
 			for _, th := range dbp.threads {
@@ -746,7 +759,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 	switchTrapthread := false
 
 	for _, dbp := range procgrp.procs {
-		if dbp.exited {
+		if ok, _ := dbp.Valid(); !ok {
 			continue
 		}
 		err := stop1(cctx, dbp, trapthread, &switchTrapthread)
@@ -759,7 +772,7 @@ func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *na
 		trapthreadID := trapthread.ID
 		trapthread = nil
 		for _, dbp := range procgrp.procs {
-			if dbp.exited {
+			if ok, _ := dbp.Valid(); !ok {
 				continue
 			}
 			for _, th := range dbp.threads {
@@ -844,6 +857,10 @@ func stop1(cctx *proc.ContinueOnceContext, dbp *nativeProcess, trapthread *nativ
 	return err1
 }
 
+func (procgrp *processGroup) detachChild(dbp *nativeProcess) error {
+	return procgrp.Detach(dbp.pid, false)
+}
+
 func (dbp *nativeProcess) detach(kill bool) error {
 	for threadID := range dbp.threads {
 		err := ptraceDetach(threadID, 0)
@@ -895,7 +912,7 @@ func (dbp *nativeProcess) SetUProbe(fnName string, goidOffset int64, args []ebpf
 
 	fns := dbp.bi.LookupFunc()[fnName]
 	if len(fns) != 1 {
-		return fmt.Errorf("could not find function: %s", fnName)
+		return &proc.ErrFunctionNotFound{FuncName: fnName}
 	}
 	fn := fns[0]
 
