@@ -10,6 +10,7 @@ import (
 
 	"github.com/go-delve/delve/pkg/dwarf/op"
 	"github.com/go-delve/delve/pkg/goversion"
+	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 )
 
@@ -36,25 +37,18 @@ const (
 // Target represents the process being debugged.
 type Target struct {
 	Process
-	RecordingManipulation
 
 	proc   ProcessInternal
 	recman RecordingManipulationInternal
 
-	pid int
+	pid     int
+	CmdLine string
 
 	// StopReason describes the reason why the target process is stopped.
 	// A process could be stopped for multiple simultaneous reasons, in which
 	// case only one will be reported.
 	StopReason StopReason
 
-	// CanDump is true if core dumping is supported.
-	CanDump bool
-
-	// KeepSteppingBreakpoints determines whether certain stop reasons (e.g. manual halts)
-	// will keep the stepping breakpoints instead of clearing them.
-	KeepSteppingBreakpoints KeepSteppingBreakpoints
-
 	// currentThread is the thread that will be used by next/step/stepout and to evaluate variables if no goroutine is selected.
 	currentThread Thread
 
@@ -63,7 +57,7 @@ type Target struct {
 	selectedGoroutine *G
 
 	// fncallForG stores a mapping of current active function calls.
-	fncallForG map[int]*callInjection
+	fncallForG map[int64]*callInjection
 
 	asyncPreemptChanged bool  // runtime/debug.asyncpreemptoff was changed
 	asyncPreemptOff     int64 // cached value of runtime/debug.asyncpreemptoff
@@ -84,7 +78,7 @@ type Target struct {
 	fakeMemoryRegistry    []*compositeMemory
 	fakeMemoryRegistryMap map[string]*compositeMemory
 
-	cctx *ContinueOnceContext
+	partOfGroup bool
 }
 
 type KeepSteppingBreakpoints uint8
@@ -151,15 +145,6 @@ const (
 	StopWatchpoint                     // The target process hit one or more watchpoints
 )
 
-// NewTargetConfig contains the configuration for a new Target object,
-type NewTargetConfig struct {
-	Path                string     // path of the main executable
-	DebugInfoDirs       []string   // Directories to search for split debug info
-	DisableAsyncPreempt bool       // Go 1.14 asynchronous preemption should be disabled
-	StopReason          StopReason // Initial stop reason
-	CanDump             bool       // Can create core dumps (must implement ProcessInternal.MemoryMap)
-}
-
 // DisableAsyncPreemptEnv returns a process environment (like os.Environ)
 // where asyncpreemptoff is set to 1.
 func DisableAsyncPreemptEnv() []string {
@@ -174,15 +159,15 @@ func DisableAsyncPreemptEnv() []string {
 	return env
 }
 
-// NewTarget returns an initialized Target object.
+// newTarget returns an initialized Target object.
 // The p argument can optionally implement the RecordingManipulation interface.
-func NewTarget(p ProcessInternal, pid int, currentThread Thread, cfg NewTargetConfig) (*Target, error) {
+func (grp *TargetGroup) newTarget(p ProcessInternal, pid int, currentThread Thread, path, cmdline string) (*Target, error) {
 	entryPoint, err := p.EntryPoint()
 	if err != nil {
 		return nil, err
 	}
 
-	err = p.BinInfo().LoadBinaryInfo(cfg.Path, entryPoint, cfg.DebugInfoDirs)
+	err = p.BinInfo().LoadBinaryInfo(path, entryPoint, grp.cfg.DebugInfoDirs)
 	if err != nil {
 		return nil, err
 	}
@@ -195,12 +180,10 @@ func NewTarget(p ProcessInternal, pid int, currentThread Thread, cfg NewTargetCo
 	t := &Target{
 		Process:       p,
 		proc:          p,
-		fncallForG:    make(map[int]*callInjection),
-		StopReason:    cfg.StopReason,
+		fncallForG:    make(map[int64]*callInjection),
 		currentThread: currentThread,
-		CanDump:       cfg.CanDump,
 		pid:           pid,
-		cctx:          &ContinueOnceContext{},
+		CmdLine:       cmdline,
 	}
 
 	if recman, ok := p.(RecordingManipulationInternal); ok {
@@ -208,18 +191,19 @@ func NewTarget(p ProcessInternal, pid int, currentThread Thread, cfg NewTargetCo
 	} else {
 		t.recman = &dummyRecordingManipulation{}
 	}
-	t.RecordingManipulation = t.recman
 
 	g, _ := GetG(currentThread)
 	t.selectedGoroutine = g
 
+	t.Breakpoints().Logical = grp.LogicalBreakpoints
 	t.createUnrecoveredPanicBreakpoint()
 	t.createFatalThrowBreakpoint()
+	t.createPluginOpenBreakpoint()
 
 	t.gcache.init(p.BinInfo())
 	t.fakeMemoryRegistryMap = make(map[string]*compositeMemory)
 
-	if cfg.DisableAsyncPreempt {
+	if grp.cfg.DisableAsyncPreempt {
 		setAsyncPreemptOff(t, 1)
 	}
 
@@ -268,7 +252,7 @@ func (t *Target) Valid() (bool, error) {
 // Currently only non-recorded processes running on AMD64 support
 // function calls.
 func (t *Target) SupportsFunctionCalls() bool {
-	return t.Process.BinInfo().Arch.Name == "amd64" || t.Process.BinInfo().Arch.Name == "arm64"
+	return t.Process.BinInfo().Arch.Name == "amd64" || (t.Process.BinInfo().Arch.Name == "arm64" && t.Process.BinInfo().GOOS != "windows") || t.Process.BinInfo().Arch.Name == "ppc64le"
 }
 
 // ClearCaches clears internal caches that should not survive a restart.
@@ -276,17 +260,24 @@ func (t *Target) SupportsFunctionCalls() bool {
 func (t *Target) ClearCaches() {
 	t.clearFakeMemory()
 	t.gcache.Clear()
+	t.BinInfo().moduleDataCache = nil
 	for _, thread := range t.ThreadList() {
 		thread.Common().g = nil
 	}
 }
 
-// Restart will start the process over from the location specified by the "from" locspec.
+// Restart will start the process group over from the location specified by the "from" locspec.
 // This is only useful for recorded targets.
 // Restarting of a normal process happens at a higher level (debugger.Restart).
-func (t *Target) Restart(from string) error {
-	t.ClearCaches()
-	currentThread, err := t.recman.Restart(t.cctx, from)
+func (grp *TargetGroup) Restart(from string) error {
+	if len(grp.targets) != 1 {
+		panic("multiple targets not implemented")
+	}
+	for _, t := range grp.targets {
+		t.ClearCaches()
+	}
+	t := grp.Selected
+	currentThread, err := t.recman.Restart(grp.cctx, from)
 	if err != nil {
 		return err
 	}
@@ -306,55 +297,33 @@ func (t *Target) SelectedGoroutine() *G {
 }
 
 // SwitchGoroutine will change the selected and active goroutine.
-func (p *Target) SwitchGoroutine(g *G) error {
-	if ok, err := p.Valid(); !ok {
+func (t *Target) SwitchGoroutine(g *G) error {
+	if ok, err := t.Valid(); !ok {
 		return err
 	}
 	if g == nil {
 		return nil
 	}
 	if g.Thread != nil {
-		return p.SwitchThread(g.Thread.ThreadID())
+		return t.SwitchThread(g.Thread.ThreadID())
 	}
-	p.selectedGoroutine = g
+	t.selectedGoroutine = g
 	return nil
 }
 
 // SwitchThread will change the selected and active thread.
-func (p *Target) SwitchThread(tid int) error {
-	if ok, err := p.Valid(); !ok {
+func (t *Target) SwitchThread(tid int) error {
+	if ok, err := t.Valid(); !ok {
 		return err
 	}
-	if th, ok := p.FindThread(tid); ok {
-		p.currentThread = th
-		p.selectedGoroutine, _ = GetG(p.CurrentThread())
+	if th, ok := t.FindThread(tid); ok {
+		t.currentThread = th
+		t.selectedGoroutine, _ = GetG(t.CurrentThread())
 		return nil
 	}
 	return fmt.Errorf("thread %d does not exist", tid)
 }
 
-// Detach will detach the target from the underylying process.
-// This means the debugger will no longer receive events from the process
-// we were previously debugging.
-// If kill is true then the process will be killed when we detach.
-func (t *Target) Detach(kill bool) error {
-	if !kill {
-		if t.asyncPreemptChanged {
-			setAsyncPreemptOff(t, t.asyncPreemptOff)
-		}
-		for _, bp := range t.Breakpoints().M {
-			if bp != nil {
-				err := t.ClearBreakpoint(bp.Addr)
-				if err != nil {
-					return err
-				}
-			}
-		}
-	}
-	t.StopReason = StopUnknown
-	return t.proc.Detach(kill)
-}
-
 // setAsyncPreemptOff enables or disables async goroutine preemption by
 // writing the value 'v' to runtime.debug.asyncpreemptoff.
 // A value of '1' means off, a value of '0' means on.
@@ -366,8 +335,12 @@ func setAsyncPreemptOff(p *Target, v int64) {
 	scope := globalScope(p, p.BinInfo(), p.BinInfo().Images[0], p.Memory())
 	// +rtype -var debug anytype
 	debugv, err := scope.findGlobal("runtime", "debug")
-	if err != nil || debugv.Unreadable != nil {
-		logger.Warnf("could not find runtime/debug variable (or unreadable): %v %v", err, debugv.Unreadable)
+	if err != nil {
+		logger.Warnf("could not find runtime/debug variable (or unreadable): %v", err)
+		return
+	}
+	if debugv.Unreadable != nil {
+		logger.Warnf("runtime/debug variable unreadable: %v", err, debugv.Unreadable)
 		return
 	}
 	asyncpreemptoffv, err := debugv.structMember("asyncpreemptoff") // +rtype int32
@@ -383,7 +356,7 @@ func setAsyncPreemptOff(p *Target, v int64) {
 	p.asyncPreemptChanged = true
 	p.asyncPreemptOff, _ = constant.Int64Val(asyncpreemptoffv.Value)
 
-	err = scope.setValue(asyncpreemptoffv, newConstant(constant.MakeInt64(v), scope.Mem), "")
+	err = scope.setValue(asyncpreemptoffv, newConstant(constant.MakeInt64(v), scope.BinInfo, scope.Mem), "")
 	if err != nil {
 		logger.Warnf("could not set asyncpreemptoff %v", err)
 	}
@@ -406,18 +379,31 @@ func (t *Target) createUnrecoveredPanicBreakpoint() {
 
 // createFatalThrowBreakpoint creates the a breakpoint as runtime.fatalthrow.
 func (t *Target) createFatalThrowBreakpoint() {
-	fatalpcs, err := FindFunctionLocation(t.Process, "runtime.throw", 0)
-	if err == nil {
-		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
+	setFatalThrow := func(pcs []uint64, err error) {
 		if err == nil {
-			bp.Logical.Name = FatalThrow
+			bp, err := t.SetBreakpoint(fatalThrowID, pcs[0], UserBreakpoint, nil)
+			if err == nil {
+				bp.Logical.Name = FatalThrow
+			}
 		}
 	}
-	fatalpcs, err = FindFunctionLocation(t.Process, "runtime.fatal", 0)
-	if err == nil {
-		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
-		if err == nil {
-			bp.Logical.Name = FatalThrow
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.throw", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.fatal", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.winthrow", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.fatalsignal", 0))
+}
+
+// createPluginOpenBreakpoint creates a breakpoint at the return instruction
+// of plugin.Open (if it exists) that will try to enable suspended
+// breakpoints.
+func (t *Target) createPluginOpenBreakpoint() {
+	retpcs, _ := findRetPC(t, "plugin.Open")
+	for _, retpc := range retpcs {
+		bp, err := t.SetBreakpoint(0, retpc, PluginOpenBreakpoint, nil)
+		if err != nil {
+			t.BinInfo().logger.Errorf("could not set plugin.Open breakpoint: %v", err)
+		} else {
+			bp.Breaklets[len(bp.Breaklets)-1].callback = t.pluginOpenCallback
 		}
 	}
 }
@@ -432,6 +418,7 @@ func (t *Target) CurrentThread() Thread {
 type UProbeTraceResult struct {
 	FnAddr       int
 	GoroutineID  int
+	IsRet        bool
 	InputParams  []*Variable
 	ReturnParams []*Variable
 }
@@ -447,8 +434,13 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 		v.Addr = ip.Addr
 		v.Kind = ip.Kind
 
+		if v.RealType == nil {
+			v.Unreadable = errors.New("type not supported by ebpf")
+			return v
+		}
+
 		cachedMem := CreateLoadedCachedMemory(ip.Data)
-		compMem, _ := CreateCompositeMemory(cachedMem, t.BinInfo().Arch, op.DwarfRegisters{}, ip.Pieces)
+		compMem, _ := CreateCompositeMemory(cachedMem, t.BinInfo().Arch, op.DwarfRegisters{}, ip.Pieces, ip.RealType.Common().ByteSize)
 		v.mem = compMem
 
 		// Load the value here so that we don't have to export
@@ -461,6 +453,7 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 		r := &UProbeTraceResult{}
 		r.FnAddr = tp.FnAddr
 		r.GoroutineID = tp.GoroutineID
+		r.IsRet = tp.IsRet
 		for _, ip := range tp.InputParams {
 			v := convertInputParamToVariable(ip)
 			r.InputParams = append(r.InputParams, v)
@@ -475,17 +468,17 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 }
 
 // ResumeNotify specifies a channel that will be closed the next time
-// Continue finishes resuming the target.
-func (t *Target) ResumeNotify(ch chan<- struct{}) {
-	t.cctx.ResumeChan = ch
+// Continue finishes resuming the targets.
+func (grp *TargetGroup) ResumeNotify(ch chan<- struct{}) {
+	grp.cctx.ResumeChan = ch
 }
 
-// RequestManualStop attempts to stop all the process' threads.
-func (t *Target) RequestManualStop() error {
-	t.cctx.StopMu.Lock()
-	defer t.cctx.StopMu.Unlock()
-	t.cctx.manualStopRequested = true
-	return t.proc.RequestManualStop(t.cctx)
+// RequestManualStop attempts to stop all the processes' threads.
+func (grp *TargetGroup) RequestManualStop() error {
+	grp.cctx.StopMu.Lock()
+	defer grp.cctx.StopMu.Unlock()
+	grp.cctx.manualStopRequested = true
+	return grp.Selected.proc.RequestManualStop(grp.cctx)
 }
 
 const (
@@ -499,14 +492,14 @@ const (
 // This caching is primarily done so that registerized variables don't get a
 // different address every time they are evaluated, which would be confusing
 // and leak memory.
-func (t *Target) newCompositeMemory(mem MemoryReadWriter, regs op.DwarfRegisters, pieces []op.Piece, descr *locationExpr) (int64, *compositeMemory, error) {
+func (t *Target) newCompositeMemory(mem MemoryReadWriter, regs op.DwarfRegisters, pieces []op.Piece, descr *locationExpr, size int64) (int64, *compositeMemory, error) {
 	var key string
 	if regs.CFA != 0 && len(pieces) > 0 {
 		// key is created by concatenating the location expression with the CFA,
 		// this combination is guaranteed to be unique between resumes.
 		buf := new(strings.Builder)
 		fmt.Fprintf(buf, "%#x ", regs.CFA)
-		op.PrettyPrint(buf, descr.instr)
+		op.PrettyPrint(buf, descr.instr, t.BinInfo().Arch.RegnumToString)
 		key = buf.String()
 
 		if cmem := t.fakeMemoryRegistryMap[key]; cmem != nil {
@@ -514,7 +507,7 @@ func (t *Target) newCompositeMemory(mem MemoryReadWriter, regs op.DwarfRegisters
 		}
 	}
 
-	cmem, err := newCompositeMemory(mem, t.BinInfo().Arch, regs, pieces)
+	cmem, err := newCompositeMemory(mem, t.BinInfo().Arch, regs, pieces, size)
 	if err != nil {
 		return 0, cmem, err
 	}
@@ -582,6 +575,30 @@ func (t *Target) dwrapUnwrap(fn *Function) *Function {
 	return fn
 }
 
+func (t *Target) pluginOpenCallback(Thread, *Target) (bool, error) {
+	logger := logflags.DebuggerLogger()
+	for _, lbp := range t.Breakpoints().Logical {
+		if isSuspended(t, lbp) {
+			err := enableBreakpointOnTarget(t, lbp)
+			if err != nil {
+				logger.Debugf("could not enable breakpoint %d: %v", lbp.LogicalID, err)
+			} else {
+				logger.Debugf("suspended breakpoint %d enabled", lbp.LogicalID)
+			}
+		}
+	}
+	return false, nil
+}
+
+func isSuspended(t *Target, lbp *LogicalBreakpoint) bool {
+	for _, bp := range t.Breakpoints().M {
+		if bp.LogicalID() == lbp.LogicalID {
+			return false
+		}
+	}
+	return true
+}
+
 type dummyRecordingManipulation struct {
 }
 
@@ -620,3 +637,9 @@ func (*dummyRecordingManipulation) ClearCheckpoint(int) error { return ErrNotRec
 func (*dummyRecordingManipulation) Restart(*ContinueOnceContext, string) (Thread, error) {
 	return nil, ErrNotRecorded
 }
+
+var ErrWaitForNotImplemented = errors.New("waitfor not implemented")
+
+func (waitFor *WaitFor) Valid() bool {
+	return waitFor != nil && waitFor.Name != ""
+}
