@@ -73,7 +73,7 @@ func (c *Client) ExpectMessage(t *testing.T) dap.Message {
 func (c *Client) ExpectInvisibleErrorResponse(t *testing.T) *dap.ErrorResponse {
 	t.Helper()
 	er := c.ExpectErrorResponse(t)
-	if er.Body.Error.ShowUser {
+	if er.Body.Error != nil && er.Body.Error.ShowUser {
 		t.Errorf("\ngot %#v\nwant ShowUser=false", er)
 	}
 	return er
@@ -82,7 +82,7 @@ func (c *Client) ExpectInvisibleErrorResponse(t *testing.T) *dap.ErrorResponse {
 func (c *Client) ExpectVisibleErrorResponse(t *testing.T) *dap.ErrorResponse {
 	t.Helper()
 	er := c.ExpectErrorResponse(t)
-	if !er.Body.Error.ShowUser {
+	if er.Body.Error == nil || !er.Body.Error.ShowUser {
 		t.Errorf("\ngot %#v\nwant ShowUser=true", er)
 	}
 	return er
@@ -91,6 +91,10 @@ func (c *Client) ExpectVisibleErrorResponse(t *testing.T) *dap.ErrorResponse {
 func (c *Client) ExpectErrorResponseWith(t *testing.T, id int, message string, showUser bool) *dap.ErrorResponse {
 	t.Helper()
 	er := c.ExpectErrorResponse(t)
+	if er.Body.Error == nil {
+		t.Errorf("got nil, want Id=%d Format=%q ShowUser=%v", id, message, showUser)
+		return er
+	}
 	if matched, _ := regexp.MatchString(message, er.Body.Error.Format); !matched || er.Body.Error.Id != id || er.Body.Error.ShowUser != showUser {
 		t.Errorf("got %#v, want Id=%d Format=%q ShowUser=%v", er, id, message, showUser)
 	}
@@ -105,7 +109,6 @@ func (c *Client) ExpectInitializeResponseAndCapabilities(t *testing.T) *dap.Init
 		SupportsConfigurationDoneRequest: true,
 		SupportsConditionalBreakpoints:   true,
 		SupportsDelayedStackTraceLoading: true,
-		SupportTerminateDebuggee:         true,
 		SupportsExceptionInfoRequest:     true,
 		SupportsSetVariable:              true,
 		SupportsFunctionBreakpoints:      true,
@@ -115,6 +118,7 @@ func (c *Client) ExpectInitializeResponseAndCapabilities(t *testing.T) *dap.Init
 		SupportsSteppingGranularity:      true,
 		SupportsLogPoints:                true,
 		SupportsDisassembleRequest:       true,
+		SupportsRestartRequest:           true,
 	}
 	if !reflect.DeepEqual(initResp.Body, wantCapabilities) {
 		t.Errorf("capabilities in initializeResponse: got %+v, want %v", pretty(initResp.Body), pretty(wantCapabilities))
@@ -137,6 +141,15 @@ func (c *Client) ExpectUnsupportedCommandErrorResponse(t *testing.T) *dap.ErrorR
 	return c.ExpectErrorResponseWith(t, 9999, "Unsupported command", false)
 }
 
+func (c *Client) ExpectCapabilitiesEventSupportTerminateDebuggee(t *testing.T) *dap.CapabilitiesEvent {
+	t.Helper()
+	e := c.ExpectCapabilitiesEvent(t)
+	if !e.Body.Capabilities.SupportTerminateDebuggee {
+		t.Errorf("\ngot %#v\nwant SupportTerminateDebuggee=true", e.Body.Capabilities.SupportTerminateDebuggee)
+	}
+	return e
+}
+
 func (c *Client) ExpectOutputEventRegex(t *testing.T, want string) *dap.OutputEvent {
 	t.Helper()
 	e := c.ExpectOutputEvent(t)
@@ -186,15 +199,28 @@ func (c *Client) ExpectOutputEventClosingClient(t *testing.T, status string) *da
 	return c.ExpectOutputEventRegex(t, fmt.Sprintf(ClosingClient, status))
 }
 
-func (c *Client) CheckStopLocation(t *testing.T, thread int, name string, line int) {
+func (c *Client) CheckStopLocation(t *testing.T, thread int, name string, line interface{}) {
 	t.Helper()
 	c.StackTraceRequest(thread, 0, 20)
 	st := c.ExpectStackTraceResponse(t)
 	if len(st.Body.StackFrames) < 1 {
 		t.Errorf("\ngot  %#v\nwant len(stackframes) => 1", st)
 	} else {
-		if line != -1 && st.Body.StackFrames[0].Line != line {
-			t.Errorf("\ngot  %#v\nwant Line=%d", st, line)
+		switch line := line.(type) {
+		case int:
+			if line != -1 && st.Body.StackFrames[0].Line != line {
+				t.Errorf("\ngot  %#v\nwant Line=%d", st, line)
+			}
+		case []int:
+			found := false
+			for _, line := range line {
+				if st.Body.StackFrames[0].Line == line {
+					found = true
+				}
+			}
+			if !found {
+				t.Errorf("\ngot  %#v\nwant Line=%v", st, line)
+			}
 		}
 		if st.Body.StackFrames[0].Name != name {
 			t.Errorf("\ngot  %#v\nwant Name=%q", st, name)
@@ -269,7 +295,9 @@ func (c *Client) DisconnectRequest() {
 // `terminateDebuggee`.
 func (c *Client) DisconnectRequestWithKillOption(kill bool) {
 	request := &dap.DisconnectRequest{Request: *c.newRequest("disconnect")}
-	request.Arguments.TerminateDebuggee = kill
+	request.Arguments = &dap.DisconnectArguments{
+		TerminateDebuggee: kill,
+	}
 	c.send(request)
 }
 
@@ -427,9 +455,13 @@ func (c *Client) TerminateRequest() {
 	c.send(&dap.TerminateRequest{Request: *c.newRequest("terminate")})
 }
 
-// RestartRequest sends a 'restart' request.
-func (c *Client) RestartRequest() {
-	c.send(&dap.RestartRequest{Request: *c.newRequest("restart")})
+// RestartRequest sends a 'restart' request with specified arguments, if provided.
+func (c *Client) RestartRequest(arguments map[string]any) {
+	request := &dap.RestartRequest{Request: *c.newRequest("restart")}
+	if arguments != nil {
+		request.Arguments = toRawMessage(arguments)
+	}
+	c.send(request)
 }
 
 // SetFunctionBreakpointsRequest sends a 'setFunctionBreakpoints' request.
@@ -548,14 +580,14 @@ func (c *Client) ReadMemoryRequest() {
 }
 
 // DisassembleRequest sends a 'disassemble' request.
-func (c *Client) DisassembleRequest(memoryReference string, instructionOffset, inctructionCount int) {
+func (c *Client) DisassembleRequest(memoryReference string, instructionOffset, instructionCount int) {
 	c.send(&dap.DisassembleRequest{
 		Request: *c.newRequest("disassemble"),
 		Arguments: dap.DisassembleArguments{
 			MemoryReference:   memoryReference,
 			Offset:            0,
 			InstructionOffset: instructionOffset,
-			InstructionCount:  inctructionCount,
+			InstructionCount:  instructionCount,
 			ResolveSymbols:    false,
 		},
 	})
