@@ -18,11 +18,14 @@ type RPCServer struct {
 	// config is all the information necessary to start the debugger and server.
 	config *service.Config
 	// debugger is a debugger service.
-	debugger *debugger.Debugger
+	debugger   *debugger.Debugger
+	eventsChan chan *proc.Event
 }
 
+const eventBufferSize = 100
+
 func NewServer(config *service.Config, debugger *debugger.Debugger) *RPCServer {
-	return &RPCServer{config, debugger}
+	return &RPCServer{config, debugger, make(chan *proc.Event, eventBufferSize)}
 }
 
 type ProcessPidIn struct {
@@ -127,7 +130,11 @@ type CommandOut struct {
 
 // Command interrupts, continues and steps through the program.
 func (s *RPCServer) Command(command api.DebuggerCommand, cb service.RPCCallback) {
-	st, err := s.debugger.Command(&command, cb.SetupDoneChan(), cb.DisconnectChan())
+	eventsFn := s.eventsFn
+	if !command.WithEvents {
+		eventsFn = nil
+	}
+	st, err := s.debugger.Command(&command, cb.SetupDoneChan(), cb.DisconnectChan(), eventsFn)
 	if err != nil {
 		cb.Return(nil, err)
 		return
@@ -137,6 +144,10 @@ func (s *RPCServer) Command(command api.DebuggerCommand, cb service.RPCCallback)
 	cb.Return(out, nil)
 }
 
+func (s *RPCServer) eventsFn(event *proc.Event) {
+	s.eventsChan <- event
+}
+
 type GetBufferedTracepointsIn struct {
 }
 
@@ -1160,3 +1171,26 @@ func (s *RPCServer) GuessSubstitutePath(arg GuessSubstitutePathIn, out *GuessSub
 	}
 	return nil
 }
+
+type GetEventsIn struct {
+}
+
+type GetEventsOut struct {
+	Events []api.Event
+}
+
+func (s *RPCServer) GetEvents(arg GetEventsIn, cb service.RPCCallback) {
+	close(cb.SetupDoneChan())
+	out := new(GetEventsOut)
+	out.Events = append(out.Events, *api.ConvertEvent(<-s.eventsChan))
+	for len(out.Events) < eventBufferSize {
+		select {
+		case event := <-s.eventsChan:
+			out.Events = append(out.Events, *api.ConvertEvent(event))
+		default:
+			cb.Return(out, nil)
+			return
+		}
+	}
+	cb.Return(out, nil)
+}
